"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var listable_format_exports = {};
__export(listable_format_exports, {
  listableFormat: () => listableFormat
});
module.exports = __toCommonJS(listable_format_exports);
var import_chalk = __toESM(require("chalk"));
var import_columnify = __toESM(require("columnify"));
var import_path = __toESM(require("path"));
var import_query_graph = require("../query-graph");
function listableFormat(pkgList, options) {
  const viewOptions = parseViewOptions(options);
  const resultList = filterResultList(pkgList, viewOptions);
  const count = resultList.length;
  let text;
  if (viewOptions.showJSON) {
    text = formatJSON(resultList);
  } else if (viewOptions.showNDJSON) {
    text = formatNDJSON(resultList);
  } else if (viewOptions.showParseable) {
    text = formatParseable(resultList, viewOptions);
  } else if (viewOptions.showGraph) {
    text = formatJSONGraph(resultList, viewOptions);
  } else {
    text = formatColumns(resultList, viewOptions);
  }
  return { text, count };
}
function parseViewOptions(options) {
  const alias = options._[0];
  return {
    showAll: alias === "la" || options.all,
    showLong: alias === "la" || alias === "ll" || options.long,
    showJSON: options.json,
    showNDJSON: options.ndjson,
    showParseable: options.parseable,
    isTopological: options.toposort,
    showGraph: options.graph
  };
}
function filterResultList(pkgList, viewOptions) {
  let result = viewOptions.showAll ? pkgList.slice() : pkgList.filter((pkg) => !pkg.private);
  if (viewOptions.isTopological) {
    result = import_query_graph.QueryGraph.toposort(result);
  }
  return result;
}
function toJSONList(resultList) {
  return resultList.map((pkg) => ({
    name: pkg.name,
    version: pkg.version,
    private: pkg.private,
    location: pkg.location
  }));
}
function formatJSON(resultList) {
  return JSON.stringify(toJSONList(resultList), null, 2);
}
function formatNDJSON(resultList) {
  return toJSONList(resultList).map((data) => JSON.stringify(data)).join("\n");
}
function formatJSONGraph(resultList, viewOptions) {
  const graph = {};
  const getNeighbors = viewOptions.showAll ? (pkg) => Object.keys(
    Object.assign(
      {},
      pkg.devDependencies,
      pkg.peerDependencies,
      pkg.optionalDependencies,
      pkg.dependencies
    )
  ).sort() : (pkg) => Object.keys(
    Object.assign(
      {},
      // no devDependencies
      // no peerDependencies
      pkg.optionalDependencies,
      pkg.dependencies
    )
  ).sort();
  for (const pkg of resultList) {
    graph[pkg.name] = getNeighbors(pkg);
  }
  return JSON.stringify(graph, null, 2);
}
function makeParseable(pkg) {
  const result = [pkg.location, pkg.name];
  if (pkg.version) {
    result.push(pkg.version);
  } else {
    result.push("MISSING");
  }
  if (pkg.private) {
    result.push("PRIVATE");
  }
  return result.join(":");
}
function formatParseable(resultList, viewOptions) {
  return resultList.map(viewOptions.showLong ? makeParseable : (pkg) => pkg.location).join("\n");
}
function getColumnOrder(viewOptions) {
  const columns = ["name"];
  if (viewOptions.showLong) {
    columns.push("version", "location");
  }
  if (viewOptions.showAll) {
    columns.push("private");
  }
  return columns;
}
function trimmedColumns(formattedResults, viewOptions) {
  const str = (0, import_columnify.default)(formattedResults, {
    showHeaders: false,
    columns: getColumnOrder(viewOptions),
    config: {
      version: {
        align: "right"
      }
    }
  });
  return str.split("\n").map((line) => line.trimRight()).join("\n");
}
function formatColumns(resultList, viewOptions) {
  const formattedResults = resultList.map((result) => {
    const formatted = {
      name: result.name
    };
    if (result.version) {
      formatted.version = import_chalk.default.green(`v${result.version}`);
    } else {
      formatted.version = import_chalk.default.yellow("MISSING");
    }
    if (result.private) {
      formatted.private = `(${import_chalk.default.red("PRIVATE")})`;
    }
    formatted.location = import_chalk.default.grey(import_path.default.relative(".", result.location));
    return formatted;
  });
  return trimmedColumns(formattedResults, viewOptions);
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  listableFormat
});
